package org.freecompany.infoset;

/**
 * An abstraction of an object that contains an arbitrary representation of an
 * XML infoset.  Implementations of this interface must be thread safe.
 */
public interface Infoset< T> {

	/**
	 * Returns the type of the infoset model object that this {@link Infoset}
	 * encompases.
	 */
	Class< T> getType();

	/**
	 * Returns the factory used to create this infoset.
	 */
	InfosetFactory< T> getFactory();

	/**
	 * Returns the representation of the infoset.  This method may be called
	 * multiple times during exection, and may be concurrently executed from
	 * multiple pipeline instances.  Therefore, the resulting objects must
	 * be thread safe in respect to itself and other like instances.
	 * <p/>
	 * The backing infoset need not be a precomputed in memory representation, though it
	 * certainly could be as in a DOM {@link org.w3c.dom.Node} infoset.  In some cases,
	 * as with the SAX {@link org.xml.sax.XMLReader} or a pull implementation the backing
	 * object may only represent a way to set a event handler for future parsing.
	 */
	T get();
}
