package org.freecompany.infoset.dom;

import java.io.InputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URL;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.freecompany.infoset.AbstractInfoset;
import org.freecompany.infoset.Infoset;
import org.freecompany.infoset.InfosetFactory;
import org.freecompany.infoset.DefaultInfoset;
import org.freecompany.infoset.InfosetException;
import org.freecompany.infoset.sax.DomWrapperInfoset;
import org.w3c.dom.Document;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.XMLFilterImpl;

import static org.freecompany.infoset.InfosetFactory.Registry.reg;
import static org.freecompany.infoset.InfosetFactory.Registry.lookup;

public class DomInfosetFactory implements InfosetFactory< Document> {

	public static DomInfosetFactory newInstance() { return new DomInfosetFactory(); }

	protected final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();

	protected DomInfosetFactory() {
		factory.setNamespaceAware( true);
	}

	public Infoset< Document> create( final URL url) throws IOException {
		return create( url.openStream());
	}

	public Infoset< Document> create( final ReadableByteChannel channel) throws IOException {
		return create( Channels.newInputStream( channel));
	}

	public Infoset< Document> create( final InputStream input) {
		if ( input == null) throw new IllegalArgumentException( "Cannot parse null input stream.");
		return new AbstractInfoset< Document>( Document.class, this) {
			private Document document;
			public synchronized Document get() throws InfosetException {
				try {
					if ( document == null) {
						DocumentBuilder builder;
						synchronized ( DomInfosetFactory.this.factory) {
							builder = DomInfosetFactory.this.factory.newDocumentBuilder();
						}
						document = builder.parse( input);
					}
				   	return document;
				} catch ( ParserConfigurationException e) {
					throw new InfosetException( e);
				} catch ( SAXException e) {
					throw new InfosetException( e);
				} catch ( IOException e) {
					throw new InfosetException( e);
				}
			}
		};
	}

	public < V> Infoset< V> adaptTo( final Class< V> clazz, final Infoset< Document> infoset) {
		if ( clazz.equals( XMLReader.class)) return ( Infoset< V>) new DomWrapperInfoset( lookup( XMLReader.class), infoset);
		return null;
	}

	public < V> Infoset< Document> adaptFrom( final Infoset< V> infoset) {
		if ( infoset.getType().equals( XMLReader.class)) {
			return new AbstractInfoset< Document>( Document.class, this) {
				public Document get() {
					// TODO: Convert SAX stream to DOM document.
					return null;
				}
			};
		}
		return null;
	}
}
